//	sdsf_pp.c
//	SDSF
//
//	Created by Nils Daumann on 25.06.11.
//	Copyright (c) 2011 Nils Daumann (www.slindev.com)

//	Permission is hereby granted, free of charge, to any person obtaining a copy
//	of this software and associated documentation files (the "Software"), to deal
//	in the Software without restriction, including without limitation the rights
//	to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//	copies of the Software, and to permit persons to whom the Software is
//	furnished to do so, subject to the following conditions:

//	The above copyright notice and this permission notice shall be included in
//	all copies or substantial portions of the Software.

//	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//	OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//	THE SOFTWARE.

#ifndef __SDSF_PP_C__
#define __SDSF_PP_C__

/**
 * Get last chain view.
 * Returns the pointer to the postprocessing chains last view.
 * @param View pointer to a view within the chain.
 * @return pointer to the chains last view.
 */
VIEW* sdsf_pp_back(VIEW* View)
{
	//find the last view of "View"s effectchain and store its pointer
	VIEW* PP_Temp_View = View;
	while(PP_Temp_View.stage != NULL)
	{
		PP_Temp_View = PP_Temp_View.stage;
	}
	
	//return the pointer
	return(PP_Temp_View);
}

/**
 * Add material to chain.
 * Adds the given material to the end of the views postprocessing chain.
 * @param View pointer to a view within the chain.
 * @param Material pointer to the material to add.
 * @return pointer to the newly created last view.
 */
VIEW* sdsf_pp_push(VIEW* View, MATERIAL* Material)
{
	//find the last view of "View"s effectchain and store its pointer
	VIEW* PP_Temp_View = View;
	while(PP_Temp_View.stage != NULL)
	{
		PP_Temp_View = PP_Temp_View.stage;
	}
	
	//create a new view as the stored views stage
	PP_Temp_View.stage = view_create(0);
	set(PP_Temp_View.stage,PROCESS_TARGET);
	
	//assign "Material" to the just created view
	PP_Temp_View = PP_Temp_View.stage;
	PP_Temp_View.material = Material;
	
	//return the pointer to the new view
	return(PP_Temp_View);
}

/**
 * Add material to chain.
 * Adds the given material to the end of the views postprocessing chain.
 * @param View pointer to a view within the chain.
 * @param Material pointer to the material to add.
 * @param target0 pointer to a texture the new view is supposed to render to.
 * @return pointer to the newly created last view.
 */
VIEW* sdsf_pp_push(VIEW* View, MATERIAL* Material, BMAP* target0)
{
	//find the last view of "View"s effectchain and store its pointer
	VIEW* PP_Temp_View = sdsf_pp_push(View, Material);
	
	//set render targets if given
	if(target0 != NULL)
	{
		PP_Temp_View.bmap = target0;
	}
	
	//return the pointer to the new view
	return(PP_Temp_View);
}

/**
 * Add material to chain.
 * Adds the given material to the end of the views postprocessing chain.
 * @param View pointer to a view within the chain.
 * @param Material pointer to the material to add.
 * @param target0 pointer to a texture the new view is supposed to render to.
 * @param target1 pointer to a texture the new view is supposed to render to as second render target.
 * @return pointer to the newly created last view.
 */
VIEW* sdsf_pp_push(VIEW* View, MATERIAL* Material, BMAP* target0, BMAP* target1)
{
	//find the last view of "View"s effectchain and store its pointer
	VIEW* PP_Temp_View = sdsf_pp_push(View, Material);
	
	//set render targets if given
	if(target0 != NULL)
	{
		PP_Temp_View.bmap = target0;
	}
	if(target1 != NULL)
	{
		PP_Temp_View.target1 = target1;
	}
	
	//return the pointer to the new view
	return(PP_Temp_View);
}

/**
 * Add material to chain.
 * Adds the given material to the end of the views postprocessing chain.
 * @param View pointer to a view within the chain.
 * @param Material pointer to the material to add.
 * @param target0 pointer to a texture the new view is supposed to render to.
 * @param target1 pointer to a texture the new view is supposed to render to as second render target.
 * @param target2 pointer to a texture the new view is supposed to render to as third render target.
 * @return pointer to the newly created last view.
 */
VIEW* sdsf_pp_push(VIEW* View, MATERIAL* Material, BMAP* target0, BMAP* target1, BMAP* target2)
{
	//find the last view of "View"s effectchain and store its pointer
	VIEW* PP_Temp_View = sdsf_pp_push(View, Material);
	
	//set render targets if given
	if(target0 != NULL)
	{
		PP_Temp_View.bmap = target0;
	}
	if(target1 != NULL)
	{
		PP_Temp_View.target1 = target1;
	}
	if(target2 != NULL)
	{
		PP_Temp_View.target2 = target2;
	}
	
	//return the pointer to the new view
	return(PP_Temp_View);
}

/**
 * Add material to chain.
 * Adds the given material to the end of the views postprocessing chain.
 * @param View pointer to a view within the chain.
 * @param Material pointer to the material to add.
 * @param target0 pointer to a texture the new view is supposed to render to.
 * @param target1 pointer to a texture the new view is supposed to render to as second render target.
 * @param target2 pointer to a texture the new view is supposed to render to as third render target.
 * @param target2 pointer to a texture the new view is supposed to render to as fourth render target.
 * @return pointer to the newly created last view.
 */
VIEW* sdsf_pp_push(VIEW* View, MATERIAL* Material, BMAP* target0, BMAP* target1, BMAP* target2, BMAP* target3)
{
	//find the last view of "View"s effectchain and store its pointer
	VIEW* PP_Temp_View = sdsf_pp_push(View, Material);
	
	//set render targets if given
	if(target0 != NULL)
	{
		PP_Temp_View.bmap = target0;
	}
	if(target1 != NULL)
	{
		PP_Temp_View.target1 = target1;
	}
	if(target2 != NULL)
	{
		PP_Temp_View.target2 = target2;
	}
	if(target3 != NULL)
	{
		PP_Temp_View.target3 = target3;
	}
	
	//return the pointer to the new view
	return(PP_Temp_View);
}

/**
 * Remove last material from chain.
 * Removes the last material from the postprocessing chain.
 * @param View pointer to a view within the chain.
 * @return pointer to the new last view.
 */
VIEW* sdsf_pp_pop(VIEW* View)
{
	//find the last view
	VIEW* PP_Temp_View = View;
	while(PP_Temp_View.stage != NULL)
	{
		View = PP_Temp_View;
		PP_Temp_View = PP_Temp_View.stage;
	}
	
	//pass the views stage to the previous view
	View.stage = NULL;
	
	//reset the views render targets to null
	PP_Temp_View.bmap = NULL;
	PP_Temp_View.target1 = NULL;
	PP_Temp_View.target2 = NULL;
	PP_Temp_View.target3 = NULL;
	
	//remove the view
	ptr_remove(PP_Temp_View);
	
	return View;
}

/**
 * Remove material from chain.
 * Removes the given material from the postprocessing chain.
 * @param View pointer to a view within the chain (must be at position before the material to remove).
 * @param Material pointer to the material to remove from the chain.
 */
void sdsf_pp_remove(VIEW* View, MATERIAL* Material)
{
	//find the view with the material selected or "StageView" and the previous view
	VIEW* PP_Temp_View = View;
	while(PP_Temp_View.material != Material)
	{
		View = PP_Temp_View;
		PP_Temp_View = PP_Temp_View.stage;
		
		//return one if the stage doesnt exist
		if(PP_Temp_View == NULL){return;}
	}
	
	//pass the views stage to the previous view
	View.stage = PP_Temp_View.stage;
	
	//pass the render target
	View.bmap = PP_Temp_View.bmap;
	View.target1 = PP_Temp_View.target1;
	View.target2 = PP_Temp_View.target2;
	View.target3 = PP_Temp_View.target3;
	
	//reset the views render targets to null
	PP_Temp_View.bmap = NULL;
	PP_Temp_View.target1 = NULL;
	PP_Temp_View.target2 = NULL;
	PP_Temp_View.target3 = NULL;
	
	//remove the view
	ptr_remove(PP_Temp_View);
}

/**
 * Remove view from chain.
 * Removes the given view from the postprocessing chain and destroys it.
 * @param View pointer to a view within the chain (must be at position before the view to remove).
 * @param StageView pointer to the view to remove from the chain.
 */
void sdsf_pp_remove(VIEW* View, VIEW* StageView)
{
	//find the view with the material selected or "StageView" and the previous view
	VIEW* PP_Temp_View = View;
	while(PP_Temp_View != StageView)
	{
		View = PP_Temp_View;
		PP_Temp_View = PP_Temp_View.stage;
		
		//return one if the stage doesnt exist
		if(PP_Temp_View == NULL){return;}
	}
	
	//pass the views stage to the previous view
	View.stage = PP_Temp_View.stage;
	
	//pass the render target
	View.bmap = PP_Temp_View.bmap;
	View.target1 = PP_Temp_View.target1;
	View.target2 = PP_Temp_View.target2;
	View.target3 = PP_Temp_View.target3;
	
	//reset the views render targets to null
	PP_Temp_View.bmap = NULL;
	PP_Temp_View.target1 = NULL;
	PP_Temp_View.target2 = NULL;
	PP_Temp_View.target3 = NULL;
	
	//remove the view
	ptr_remove(PP_Temp_View);
}

/**
 * Destroy chain.
 * Destroys all views of the chain after the specified view.
 * @param View pointer to a view within the chain (must be the first view to destroy the whole chain).
 */
void sdsf_pp_clear(VIEW* View)
{
	if(View.stage == NULL){return;}
	VIEW* PP_Temp_View = View.stage;
	View.stage = NULL;
	View.bmap = NULL;
	while(PP_Temp_View.stage != NULL)
	{
		View = PP_Temp_View;
		PP_Temp_View = View.stage;
		
		//pass the views stage to the previous view
		View.stage = NULL;
		
		//reset the views render targets to null
		View.bmap = NULL;
		View.target1 = NULL;
		View.target2 = NULL;
		View.target3 = NULL;
		
		//remove the view
		ptr_remove(View);
	}
}

#endif